module Adminbox
  class Command
    attr_reader :encoded_jwt

    def initialize(jwt)
      @encoded_jwt = jwt
    end

    def execute
      subcommand = Object.const_get("Commands::#{table_module}::#{name}")
      subcommand.new(encoded_jwt).execute
    end

    def shared_secret
      Adminbox.configuration.shared_secret
    end

    def decoded_jwt
      @decoded_jwt ||= begin
                         JWT.decode(encoded_jwt, shared_secret)
                       rescue JWT::VerificationError
                         raise Adminbox::InvalidSignature
                       rescue JWT::ExpiredSignature
                         raise Adminbox::ExpiredSignature
                       end
    end

    def payload
      decoded_jwt[0]
    end

    def headers
      decoded_jwt[1]
    end

    def name
      payload["command"]
    end

    def table
      payload["table"]
    end

    def table_module
      table.split('_').collect(&:capitalize).join
    end

    def ids
      payload["ids"]
    end
  end
end
